/*
 * STRUMPACK -- STRUctured Matrices PACKage, Copyright (c) 2014, The Regents of
 * the University of California, through Lawrence Berkeley National Laboratory
 * (subject to receipt of any required approvals from the U.S. Dept. of Energy).
 * All rights reserved.
 *
 * If you have questions about your rights to use or distribute this software,
 * please contact Berkeley Lab's Technology Transfer Department at TTD@lbl.gov.
 *
 * NOTICE. This software is owned by the U.S. Department of Energy. As such, the
 * U.S. Government has been granted for itself and others acting on its behalf a
 * paid-up, nonexclusive, irrevocable, worldwide license in the Software to
 * reproduce, prepare derivative works, and perform publicly and display publicly.
 * Beginning five (5) years after the date permission to assert copyright is
 * obtained from the U.S. Department of Energy, and subject to any subsequent five
 * (5) year renewals, the U.S. Government is granted for itself and others acting
 * on its behalf a paid-up, nonexclusive, irrevocable, worldwide license in the
 * Software to reproduce, prepare derivative works, distribute copies to the
 * public, perform publicly and display publicly, and to permit others to do so.
 *
 * Developers: Francois-Henry Rouet, Xiaoye S. Li, Pieter Ghysels
 *             (Lawrence Berkeley National Lab, Computational Research Division).
 *
 */

#ifndef BLAS_HPP
#define BLAS_HPP

/* BLAS routines */

  /* xGEMM*/
  template<typename T> inline void gemm(char, char, int, int, int, T, T *, int, T *, int, T, T *, int);
  template<> inline void gemm<double>  (char transA, char transB, int m, int n, int k,   double alpha,   double *A, int lda,   double *B, int ldb,   double beta,   double *C, int ldc) {
    dgemm_(&transA,&transB,&m,&n,&k,&alpha,A,&lda,B,&ldb,&beta,C,&ldc);
  }
  template<> inline void gemm<float>   (char transA, char transB, int m, int n, int k,    float alpha,    float *A, int lda,    float *B, int ldb,    float beta,    float *C, int ldc) {
    sgemm_(&transA,&transB,&m,&n,&k,&alpha,A,&lda,B,&ldb,&beta,C,&ldc);
  }
  template<> inline void gemm<dcomplex>(char transA, char transB, int m, int n, int k, dcomplex alpha, dcomplex *A, int lda, dcomplex *B, int ldb, dcomplex beta, dcomplex *C, int ldc) {
    zgemm_(&transA,&transB,&m,&n,&k,&alpha,A,&lda,B,&ldb,&beta,C,&ldc);
  }
  template<> inline void gemm<scomplex>(char transA, char transB, int m, int n, int k, scomplex alpha, scomplex *A, int lda, scomplex *B, int ldb, scomplex beta, scomplex *C, int ldc) {
    cgemm_(&transA,&transB,&m,&n,&k,&alpha,A,&lda,B,&ldb,&beta,C,&ldc);
  }

  /* xNRM2 */
  template<typename T, typename S> inline S nrm2(int, T*, int);
  template<> inline double nrm2<  double,double>(int n,   double *X, int incx) {
    return dnrm2_ (&n,X,&incx);
  }
  template<> inline  float nrm2<   float, float>(int n,    float *X, int incx) {
    return snrm2_ (&n,X,&incx);
  }
  template<> inline double nrm2<dcomplex,double>(int n, dcomplex *X, int incx) {
    return dznrm2_(&n,X,&incx);
  }
  template<> inline  float nrm2<scomplex, float>(int n, scomplex *X, int incx) {
    return scnrm2_(&n,X,&incx);
  }

  /* xSCAL */
  template<typename T> inline void scal(int, T, T*, int);
  template<> inline void scal<  double>(int N,   double alpha,   double* X, int incx) {
    dscal_(&N,&alpha,X,&incx);
  }
  template<> inline void scal<   float>(int N,    float alpha,    float* X, int incx) {
    sscal_(&N,&alpha,X,&incx);
  }
  template<> inline void scal<dcomplex>(int N, dcomplex alpha, dcomplex* X, int incx) {
    zscal_(&N,&alpha,X,&incx);
  }
  template<> inline void scal<scomplex>(int N, scomplex alpha, scomplex* X, int incx) {
    cscal_(&N,&alpha,X,&incx);
  }

  /* xGEMV */
  template<typename T> inline void gemv(char, int, int, T, T*, int, T*, int, T, T*, int);
  template<> inline void gemv<  double>(char transA, int m, int n,   double alpha,   double *A, int lda,   double *X, int incx,   double beta,   double *Y, int incy){
    dgemv_(&transA,&m,&n,&alpha,A,&lda,X,&incx,&beta,Y,&incy);
  }
  template<> inline void gemv<   float>(char transA, int m, int n,    float alpha,    float *A, int lda,    float *X, int incx,    float beta,    float *Y, int incy){
    sgemv_(&transA,&m,&n,&alpha,A,&lda,X,&incx,&beta,Y,&incy);
  }
  template<> inline void gemv<dcomplex>(char transA, int m, int n, dcomplex alpha, dcomplex *A, int lda, dcomplex *X, int incx, dcomplex beta, dcomplex *Y, int incy){
    zgemv_(&transA,&m,&n,&alpha,A,&lda,X,&incx,&beta,Y,&incy);
  }
  template<> inline void gemv<scomplex>(char transA, int m, int n, scomplex alpha, scomplex *A, int lda, scomplex *X, int incx, scomplex beta, scomplex *Y, int incy){
    cgemv_(&transA,&m,&n,&alpha,A,&lda,X,&incx,&beta,Y,&incy);
  }

  /* xTRSM */
  template<typename T> inline void trsm(char, char, char, char, int ,int, T, T*, int, T*, int);
  template<> inline void trsm<   double>(char side, char uplo, char transA, char diag, int m, int n,   double alpha,   double *A, int lda,   double *B, int ldb) {
    dtrsm_(&side,&uplo,&transA,&diag,&m,&n,&alpha,A,&lda,B,&ldb);
  }
  template<> inline void trsm<   float>(char side, char uplo, char transA, char diag, int m, int n,    float alpha,    float *A, int lda,    float *B, int ldb) {
    strsm_(&side,&uplo,&transA,&diag,&m,&n,&alpha,A,&lda,B,&ldb);
  }
  template<> inline void trsm<dcomplex>(char side, char uplo, char transA, char diag, int m, int n, dcomplex alpha, dcomplex *A, int lda, dcomplex *B, int ldb) {
    ztrsm_(&side,&uplo,&transA,&diag,&m,&n,&alpha,A,&lda,B,&ldb);
  }
  template<> inline void trsm<scomplex>(char side, char uplo, char transA, char diag, int m, int n, scomplex alpha, scomplex *A, int lda, scomplex *B, int ldb) {
    ctrsm_(&side,&uplo,&transA,&diag,&m,&n,&alpha,A,&lda,B,&ldb);
  }

  /* xGERC */
  template<typename T> inline void geru(int, int, T, T*, int, T*, int, T*, int);
  template<> inline void geru<  double>(int m, int n,   double alpha,   double *X, int incx,   double *Y, int incy,   double *A, int lda) {
    dger_ (&m,&n,&alpha,X,&incx,Y,&incy,A,&lda);
  }
  template<> inline void geru<   float>(int m, int n,    float alpha,    float *X, int incx,    float *Y, int incy,    float *A, int lda) {
    sger_ (&m,&n,&alpha,X,&incx,Y,&incy,A,&lda);
  }
  template<> inline void geru<dcomplex>(int m, int n, dcomplex alpha, dcomplex *X, int incx, dcomplex *Y, int incy, dcomplex *A, int lda) {
    zgeru_(&m,&n,&alpha,X,&incx,Y,&incy,A,&lda);
  }
  template<> inline void geru<scomplex>(int m, int n, scomplex alpha, scomplex *X, int incx, scomplex *Y, int incy, scomplex *A, int lda) {
    cgeru_(&m,&n,&alpha,X,&incx,Y,&incy,A,&lda);
  }

  /* xSWAP */
  template<typename T> inline void swap(int, T *, int, T *, int);
  template<> inline void swap<  double>(int n,   double *X, int incx,   double *Y, int incy) {
    dswap_(&n,X,&incx,Y,&incy);
  }
  template<> inline void swap<   float>(int n,    float *X, int incx,    float *Y, int incy) {
    sswap_(&n,X,&incx,Y,&incy);
  }
  template<> inline void swap<dcomplex>(int n, dcomplex *X, int incx, dcomplex *Y, int incy) {
    zswap_(&n,X,&incx,Y,&incy);
  }
  template<> inline void swap<scomplex>(int n, scomplex *X, int incx, scomplex *Y, int incy) {
    cswap_(&n,X,&incx,Y,&incy);
  }

#endif // BLAS_HPP
