/*
 * STRUMPACK -- STRUctured Matrices PACKage, Copyright (c) 2014, The Regents of
 * the University of California, through Lawrence Berkeley National Laboratory
 * (subject to receipt of any required approvals from the U.S. Dept. of Energy).
 * All rights reserved.
 *
 * If you have questions about your rights to use or distribute this software,
 * please contact Berkeley Lab's Technology Transfer Department at TTD@lbl.gov.
 *
 * NOTICE. This software is owned by the U.S. Department of Energy. As such, the
 * U.S. Government has been granted for itself and others acting on its behalf a
 * paid-up, nonexclusive, irrevocable, worldwide license in the Software to
 * reproduce, prepare derivative works, and perform publicly and display publicly.
 * Beginning five (5) years after the date permission to assert copyright is
 * obtained from the U.S. Department of Energy, and subject to any subsequent five
 * (5) year renewals, the U.S. Government is granted for itself and others acting
 * on its behalf a paid-up, nonexclusive, irrevocable, worldwide license in the
 * Software to reproduce, prepare derivative works, distribute copies to the
 * public, perform publicly and display publicly, and to permit others to do so.
 *
 * Developers: Francois-Henry Rouet, Xiaoye S. Li, Pieter Ghysels
 *             (Lawrence Berkeley National Lab, Computational Research Division).
 *
 */

#ifndef SCALAPACK_H
#define SCALAPACK_H

/* ScaLAPACK routines */

  /* Arithmetic-independent routines */
  int numroc_(int *, int *, int *, int *, int *);
  void descinit_(int *, int *, int *, int *, int *, int *, int *, int *, int *, int *);
  void descset_(int *, int *, int *, int *, int *, int *, int *, int *, int *);
  int indxg2p_(int *, int *, int *, int *, int *);
  int indxg2l_(int *, int *, int *, int *, int *);
  int indxl2g_(int *, int *, int *, int *, int *);
  void igamn2d_(int *, const char *, const char *, int *, int *, int *, int *, int *, int *, int *, int *, int *);

  /* xGEBS2D */
  void igebs2d_(int *, const char *, const char *, int *, int *,      int *, int *);
  void dgebs2d_(int *, const char *, const char *, int *, int *,   double *, int *);
  void sgebs2d_(int *, const char *, const char *, int *, int *,    float *, int *);
  void zgebs2d_(int *, const char *, const char *, int *, int *, dcomplex *, int *);
  void cgebs2d_(int *, const char *, const char *, int *, int *, scomplex *, int *);

  /* xGEBR2D */
  void igebr2d_(int *, const char *, const char *, int *, int *,      int *, int *, int *, int *);
  void dgebr2d_(int *, const char *, const char *, int *, int *,   double *, int *, int *, int *);
  void sgebr2d_(int *, const char *, const char *, int *, int *,    float *, int *, int *, int *);
  void zgebr2d_(int *, const char *, const char *, int *, int *, dcomplex *, int *, int *, int *);
  void cgebr2d_(int *, const char *, const char *, int *, int *, scomplex *, int *, int *, int *);

  /* xGSUM2D */
  void dgsum2d_(int *, char *, char *, int *, int *, double *, int *, int *, int *);
  void sgsum2d_(int *, char *, char *, int *, int *,  float *, int *, int *, int *);

  /* xGAMX2D */
  void dgamx2d_(int *, char *, char *, int *, int *, double *, int *, int *, int *, int *, int *, int *);
  void sgamx2d_(int *, char *, char *, int *, int *,  float *, int *, int *, int *, int *, int *, int *);

  /* PxAMAX */
  void pdamax_(int *,   double *, int *,   double *, int *, int *, int *, int *);
  void psamax_(int *,    float *, int *,    float *, int *, int *, int *, int *);
  void pzamax_(int *, dcomplex *, int *, dcomplex *, int *, int *, int *, int *);
  void pcamax_(int *, scomplex *, int *, scomplex *, int *, int *, int *, int *);

  /* PxSWAP */
  void pdswap_ (int *,   double *, int *, int *, int *, int *,   double *, int *, int *, int *, int *);
  void psswap_ (int *,    float *, int *, int *, int *, int *,    float *, int *, int *, int *, int *);
  void pzswap_ (int *, dcomplex *, int *, int *, int *, int *, dcomplex *, int *, int *, int *, int *);
  void pcswap_ (int *, scomplex *, int *, int *, int *, int *, scomplex *, int *, int *, int *, int *);

  /* PxSCAL */
  void pdscal_(int *,   double *,   double *, int *, int *, int *, int *);
  void psscal_(int *,    float *,    float *, int *, int *, int *, int *);
  void pzscal_(int *, dcomplex *, dcomplex *, int *, int *, int *, int *);
  void pcscal_(int *, scomplex *, scomplex *, int *, int *, int *, int *);

  /* PxGEMV */
  void pdgemv_(char *, int *, int *,   double *,   double *, int *, int *, int *,   double *, int *, int *, int *, int *,   double *,   double *, int *, int *, int *, int *);
  void psgemv_(char *, int *, int *,    float *,    float *, int *, int *, int *,    float *, int *, int *, int *, int *,    float *,    float *, int *, int *, int *, int *);
  void pzgemv_(char *, int *, int *, dcomplex *, dcomplex *, int *, int *, int *, dcomplex *, int *, int *, int *, int *, dcomplex *, dcomplex *, int *, int *, int *, int *);
  void pcgemv_(char *, int *, int *, scomplex *, scomplex *, int *, int *, int *, scomplex *, int *, int *, int *, int *, scomplex *, scomplex *, int *, int *, int *, int *);

  /* PxGEMM */
  void pdgemm_(char *, char *, int *, int *, int *,   double *,   double *, int *, int *, int *,   double *, int *, int *, int *,   double *,   double *, int *, int *, int *);
  void psgemm_(char *, char *, int *, int *, int *,    float *,    float *, int *, int *, int *,    float *, int *, int *, int *,    float *,    float *, int *, int *, int *);
  void pzgemm_(char *, char *, int *, int *, int *, dcomplex *, dcomplex *, int *, int *, int *, dcomplex *, int *, int *, int *, dcomplex *, dcomplex *, int *, int *, int *);
  void pcgemm_(char *, char *, int *, int *, int *, scomplex *, scomplex *, int *, int *, int *, scomplex *, int *, int *, int *, scomplex *, scomplex *, int *, int *, int *);

  /* PxLACGV */
  void pzlacgv_(int*, dcomplex *X, int *, int *, int *, int *);
  void pclacgv_(int*, scomplex *X, int *, int *, int *, int *);

  /* PxGER */
  void pdger_ (int *, int *,   double *,   double *, int *, int *, int *, int *,     double *, int *, int *, int *, int *, double *, int *, int *, int *);
  void psger_ (int *, int *,    float *,    float *, int *, int *, int *, int *,    float *, int *, int *, int *, int *,    float *, int *, int *, int *);
  void pzgeru_(int *, int *, dcomplex *, dcomplex *, int *, int *, int *, int *, dcomplex *, int *, int *, int *, int *, dcomplex *, int *, int *, int *);
  void pcgeru_(int *, int *, scomplex *, scomplex *, int *, int *, int *, int *, scomplex *, int *, int *, int *, int *, scomplex *, int *, int *, int *);

  /* PxLASWP */
  void pdlaswp_(char *, char *, int *,   double *, int *, int *, int *, int *, int *, int *);
  void pslaswp_(char *, char *, int *,    float *, int *, int *, int *, int *, int *, int *);
  void pzlaswp_(char *, char *, int *, dcomplex *, int *, int *, int *, int *, int *, int *);
  void pclaswp_(char *, char *, int *, scomplex *, int *, int *, int *, int *, int *, int *);

  /* PxLAPIV */
  void pdlapiv_(char *, char *, char *, int *, int *,   double *, int *, int *, int *, int *, int *, int *, int *, int *);
  void pslapiv_(char *, char *, char *, int *, int *,    float *, int *, int *, int *, int *, int *, int *, int *, int *);
  void pzlapiv_(char *, char *, char *, int *, int *, dcomplex *, int *, int *, int *, int *, int *, int *, int *, int *);
  void pclapiv_(char *, char *, char *, int *, int *, scomplex *, int *, int *, int *, int *, int *, int *, int *, int *);

  /* PxTRSM */
  void pdtrsm_(char *, char *, char *, char *, int *, int *,   double *,   double *, int *, int *, int *,   double *, int *, int *, int *);
  void pstrsm_(char *, char *, char *, char *, int *, int *,    float *,    float *, int *, int *, int *,    float *, int *, int *, int *);
  void pztrsm_(char *, char *, char *, char *, int *, int *, dcomplex *, dcomplex *, int *, int *, int *, dcomplex *, int *, int *, int *);
  void pctrsm_(char *, char *, char *, char *, int *, int *, scomplex *, scomplex *, int *, int *, int *, scomplex *, int *, int *, int *);

  /* PxTRSV */
  void pdtrsv_(char *, char *, char *, int *,   double *, int *, int *, int *,   double *, int *, int *, int *, int *);
  void pstrsv_(char *, char *, char *, int *,    float *, int *, int *, int *,    float *, int *, int *, int *, int *);
  void pztrsv_(char *, char *, char *, int *, dcomplex *, int *, int *, int *, dcomplex *, int *, int *, int *, int *);
  void pctrsv_(char *, char *, char *, int *, scomplex *, int *, int *, int *, scomplex *, int *, int *, int *, int *);

  /* PxLANGE */
  double pdlange_(char*, int*, int*,   double*, int*, int*, int*, double*);
   float pslange_(char*, int*, int*,    float*, int*, int*, int*,  float*);
  double pzlange_(char*, int*, int*, dcomplex*, int*, int*, int*, double*);
   float pclange_(char*, int*, int*, scomplex*, int*, int*, int*,  float*);

  /* PxGEADD */
  void pdgeadd_(char *, int *, int *,   double *,   double *, int *, int *, int *,   double *,   double *, int *, int *, int *);
  void psgeadd_(char *, int *, int *,    float *,    float *, int *, int *, int *,    float *,    float *, int *, int *, int *);
  void pzgeadd_(char *, int *, int *, dcomplex *, dcomplex *, int *, int *, int *, dcomplex *, dcomplex *, int *, int *, int *);
  void pcgeadd_(char *, int *, int *, scomplex *, scomplex *, int *, int *, int *, scomplex *, scomplex *, int *, int *, int *);

  /* PxLACPY */
  void pdlacpy_(char *, int *, int *,   double *, int *, int *, int *,   double *, int *, int *, int *);
  void pslacpy_(char *, int *, int *,    float *, int *, int *, int *,    float *, int *, int *, int *);
  void pzlacpy_(char *, int *, int *, dcomplex *, int *, int *, int *, dcomplex *, int *, int *, int *);
  void pclacpy_(char *, int *, int *, scomplex *, int *, int *, int *, scomplex *, int *, int *, int *);

  /* pxGEMR2D */
  void pdgemr2d_(int *, int *,   double *, int *, int *, int *,   double *, int *, int *, int *, int *);
  void psgemr2d_(int *, int *,    float *, int *, int *, int *,    float *, int *, int *, int *, int *);
  void pzgemr2d_(int *, int *, dcomplex *, int *, int *, int *, dcomplex *, int *, int *, int *, int *);
  void pcgemr2d_(int *, int *, scomplex *, int *, int *, int *, scomplex *, int *, int *, int *, int *);

  /* PxTRAN */
  void pdtran_(int *, int *,   double *,   double *, int *, int *, int *,   double *,   double *, int *, int *, int *);
  void pstran_(int *, int *,    float *,    float *, int *, int *, int *,    float *,    float *, int *, int *, int *);
  void pztranc_(int *, int *, dcomplex *, dcomplex *, int *, int *, int *, dcomplex *, dcomplex *, int *, int *, int *);
  void pctranc_(int *, int *, scomplex *, scomplex *, int *, int *, int *, scomplex *, scomplex *, int *, int *, int *);

  /* PxGEQPF; this one is used only in the examples */
  void pdgeqpf_(int *, int *,   double *, int *, int *, int *, int *,   double *,   double *, int *, int *);
  void psgeqpf_(int *, int *,    float *, int *, int *, int *, int *,    float *,    float *, int *, int *);
  void pzgeqpf_(int *, int *, dcomplex *, int *, int *, int *, int *, dcomplex *, dcomplex *, int *, double *, int *, int *);
  void pcgeqpf_(int *, int *, scomplex *, int *, int *, int *, int *, scomplex *, scomplex *, int *,  float *, int *, int *);

  /* PxGEQPFmod */
  void pdgeqpfmod_(int *, int *,   double *, int *, int *, int *, int *,   double *,   double *, int *, int *, int *, int *, int *, double *);
  void psgeqpfmod_(int *, int *,    float *, int *, int *, int *, int *,    float *,    float *, int *, int *, int *, int *, int *,  float *);
  void pzgeqpfmod_(int *, int *, dcomplex *, int *, int *, int *, int *, dcomplex *, dcomplex *, int *, double *, int *, int *, int *, int *, int *, double *);
  void pcgeqpfmod_(int *, int *, scomplex *, int *, int *, int *, int *, scomplex *, scomplex *, int *,  float *, int *, int *, int *, int *, int *,  float *);

  /* PxGETRF */
  void pdgetrf_(int *, int *,   double *, int *, int *, int *, int *, int *);
  void psgetrf_(int *, int *,    float *, int *, int *, int *, int *, int *);
  void pzgetrf_(int *, int *, dcomplex *, int *, int *, int *, int *, int *);
  void pcgetrf_(int *, int *, scomplex *, int *, int *, int *, int *, int *);

  /* PxGETRS */
  void pdgetrs_(char *, int *, int *,   double *, int *, int *, int *, int *,   double *, int *, int *, int *, int *);
  void psgetrs_(char *, int *, int *,    float *, int *, int *, int *, int *,    float *, int *, int *, int *, int *);
  void pzgetrs_(char *, int *, int *, dcomplex *, int *, int *, int *, int *, dcomplex *, int *, int *, int *, int *);
  void pcgetrs_(char *, int *, int *, scomplex *, int *, int *, int *, int *, scomplex *, int *, int *, int *, int *);

  /* PxGELQF */
  void pdgelqf_(int *, int *,   double *, int *, int *, int *,   double *,   double *, int *, int *);
  void psgelqf_(int *, int *,    float *, int *, int *, int *,    float *,    float *, int *, int *);
  void pzgelqf_(int *, int *, dcomplex *, int *, int *, int *, dcomplex *, dcomplex *, int *, int *);
  void pcgelqf_(int *, int *, scomplex *, int *, int *, int *, scomplex *, scomplex *, int *, int *);

  /* PxxxGLQ */
  void pdorglq_(int *, int *, int *,   double *, int *, int *, int *,   double *,   double *, int *, int *);
  void psorglq_(int *, int *, int *,    float *, int *, int *, int *,    float *,    float *, int *, int *);
  void pzunglq_(int *, int *, int *, dcomplex *, int *, int *, int *, dcomplex *, dcomplex *, int *, int *);
  void pcunglq_(int *, int *, int *, scomplex *, int *, int *, int *, scomplex *, scomplex *, int *, int *);

  /* PxNRM2; this one is used only in the examples */
  void pdnrm2_(int *,  double *,   double *, int *, int *, int *, int *);
  void psnrm2_(int *,   float *,    float *, int *, int *, int *, int *);
  void pdznrm2_(int *, double *, dcomplex *, int *, int *, int *, int *);
  void pscnrm2_(int *,  float *, scomplex *, int *, int *, int *, int *);

#endif // SCALAPACK_H
