/*
 * STRUMPACK -- STRUctured Matrices PACKage, Copyright (c) 2014, The Regents of
 * the University of California, through Lawrence Berkeley National Laboratory
 * (subject to receipt of any required approvals from the U.S. Dept. of Energy).
 * All rights reserved.
 *
 * If you have questions about your rights to use or distribute this software,
 * please contact Berkeley Lab's Technology Transfer Department at TTD@lbl.gov.
 *
 * NOTICE. This software is owned by the U.S. Department of Energy. As such, the
 * U.S. Government has been granted for itself and others acting on its behalf a
 * paid-up, nonexclusive, irrevocable, worldwide license in the Software to
 * reproduce, prepare derivative works, and perform publicly and display publicly.
 * Beginning five (5) years after the date permission to assert copyright is
 * obtained from the U.S. Department of Energy, and subject to any subsequent five
 * (5) year renewals, the U.S. Government is granted for itself and others acting
 * on its behalf a paid-up, nonexclusive, irrevocable, worldwide license in the
 * Software to reproduce, prepare derivative works, distribute copies to the
 * public, perform publicly and display publicly, and to permit others to do so.
 *
 * Developers: Pieter Ghysels, Francois-Henry Rouet, Xiaoye S. Li.
 *             (Lawrence Berkeley National Lab, Computational Research Division).
 *
 */
#include "strumpack_parameters.hpp"
#if defined(_OPENMP)
#include <omp.h>
#endif

namespace strumpack {

  namespace params {

    // these are all global variables

    int separator_ordering_level    = 1;
    int hss_minimum_front_size      = 512;
    int hss_minimum_partition_size  = 128;
    int hss_coarsening_factor       = 2;
    int random_vector_blocksize     = 128;
    int task_recursion_cutoff_level = 6;
    int rank_offset                 = 128;
    float rank_factor               = 1;
    int maximum_rank                = 2000;
    int mc64job                     = 5;
    int q_power                     = 0;
    bool log_etree                  = false;
    bool log_ranks                  = false;
    int random_vector_panel         = 4;

    RandomDistribution random_distribution = NORMAL;
    RandomEngine random_engine = LINEAR;

#if defined(_OPENMP)
    int num_threads = omp_get_max_threads();
#else
    int num_threads = 1;
#endif

    long long int total_mem = 0;
    long long int max_mem = 0;
    int mem_new_free = 0;

    std::atomic<long long int> flops(0);
    std::atomic<long long int> bytes(0);

  } // end namespace params

} // end namespace strumpack
